#!/usr/bin/php
<?php
# @package      hubzero-metrics
# @file         xlogimport_authlog.php
# @copyright    Copyright (c) 2011-2020 The Regents of the University of California.
# @license      http://opensource.org/licenses/MIT MIT
#
# Copyright (c) 2011-2020 The Regents of the University of California.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# HUBzero is a registered trademark of The Regents of the University of California.
#
# ------------------------------------------------------------------------- 
# This Script imports information in the auth.log file into table userlogin
#
# USAGE: ./xlogimport_authlog.php <filename>
#
# NOTES: This script handles log files in the old and new formats
# 2007-05-17 11:06:39 username 128.210.189.195 login
# 2009-01-17 11:06:39 1234 [username] 128.210.189.195 login
#

error_reporting(E_ALL & ~E_NOTICE);
@ini_set('display_errors','1');

if(!defined('__DIR__')) {
    $fPos = strrpos(__FILE__, "/");
    define("__DIR__", substr(__FILE__, 0, $fPos) . "/");
}

require_once(__DIR__."/../includes/hub_parameters.php");
require_once(__DIR__."/../includes/db_connect.php");
require_once(__DIR__."/../includes/func_misc.php");

$db_hub = db_connect('db_hub');

$unrec = '';

$filehandle = fopen($_SERVER['argv'][1], "r");
if (!$filehandle) {
    $msg = "Error opening file: ".$_SERVER['argv'][1]."\n";
    clean_exit($msg);
}

# building excluded IP list
$filtered_ips = gen_exclude_list('ip');

$log_pattern_1 = '/^(\d{4}-\d{2}-\d{2})\s+(\d{2}:\d{2}:\d{2})\s+([\d]+)\s+(\[.+\])\s+([\.\d]+)\s+([\w]+)$/';
$log_pattern_2 = '/^(\d{4}-\d{2}-\d{2})\s+(\d{2}:\d{2}:\d{2})\s+(.+)\s+([\.\d]+)\s+([\w]+)$/';

while (1) {

    if (feof($filehandle))
        break;

    $line = fgets($filehandle);
    if (!$line) 
        break;

    if (preg_match($log_pattern_1, $line, $matches)) {
        $datetime = $matches[1].' '.$matches[2];
        $uidNumber = trim($matches[3]);
        $user = rtrim(ltrim(trim($matches[4]),'['),']');
        $ip = trim($matches[5]);
        $action = trim($matches[6]);
    } else if ( preg_match($log_pattern_2, $line, $matches)) {
        $datetime = $matches[1].' '.$matches[2];
        $uidNumber = 0;
        $user = trim($matches[3]);
        $ip = trim($matches[4]);
        $action = trim($matches[5]);
    } else {
        $unrec .= 'Unrecognized format: '.$line;
        continue;
    }

    if ($user == "")
        $user = '-';

    if ($user <> "hubstatus" && $user <> "hubadmin" && (!search_array($ip, $filtered_ips))) {
        $sql_ins = 'INSERT IGNORE INTO '.$metrics_db.'.userlogin (datetime, uidNumber, user, ip, action) VALUES ('.dbquote($datetime).','.dbquote($uidNumber).','.dbquote($user).','.dbquote($ip).','.dbquote($action).')';
        db_exec($db_hub, $sql_ins);
    }
}

if ($unrec)
    print $unrec;

fclose($filehandle);
mysqli_close($db_hub);

?>
