#!/usr/bin/perl -w
# @package      hubzero-metrics
# @file         xlogfix_middleware_wall.pl
# @copyright    Copyright (c) 2011-2020 The Regents of the University of California.
# @license      http://opensource.org/licenses/MIT MIT
#
# Copyright (c) 2011-2020 The Regents of the University of California.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# HUBzero is a registered trademark of The Regents of the University of California.
#
# =========================================================================
# This Script copies over the simulation tool data from middleware logs
#
# USAGE: ./xlogimport_middleware_wall.pl
#

use strict;
use DBI;

use FindBin '$Bin';
my $filename = '/etc/hubzero-metrics/access.cfg';

our $hub_db;
our $metrics_db;
our $db_host;
our $db_user;
our $db_pass;
do $filename;

my $debug;
my $dbhandle;
my $dbsthandle;
my $dbrowvalues;
my @dbrow;
my $mw_handle;
my $mw_sthandle;
my $mw_rowvalues;
my @mw_row;
my $statement;
my $dt;
my $user;
my $ip;
my $tool;
my $execunit;
my $walltime;

$debug = 0;

my $exclude_users = "'gridstat'";

#  Open database...
#-------------------
#print "Opening Middleware database...\n";
$mw_handle = DBI->connect("DBI:mysql:$hub_db:$db_host", $db_user, $db_pass);
#print "Opening usage statistics database...\n";
$dbhandle = DBI->connect("DBI:mysql:$metrics_db:$db_host", $db_user, $db_pass);

#  Select all Middleware records...
#--------------------------------
#print "Finding all Middleware records...\n";
$statement = "SELECT joblog.start, sessionlog.username, sessionlog.remoteip, sessionlog.appname, sessionlog.exechost, joblog.walltime FROM joblog, sessionlog WHERE joblog.sessnum = sessionlog.sessnum AND sessionlog.username NOT IN (".$exclude_users.") AND sessionlog.username NOT LIKE 'hctest%' AND joblog.event <> '[waiting]' ORDER BY joblog.start, sessionlog.username, sessionlog.remoteip";
$mw_sthandle = $mw_handle->prepare($statement)
    or die "Error:  can't prepare statement \"$statement\" ($mw_handle->errstr).\n";
$mw_rowvalues = $mw_sthandle->execute
    or die "Error:  can't execute the query ($mw_sthandle->errstr).\n";

#  Select all toolstart records already imported from Middleware...
#----------------------------------------------------------------
#print "Finding all toolstart records already imported from Middleware...\n";
$statement = "SELECT id, datetime, user, ip, tool, execunit, walltime FROM toolstart ORDER BY datetime, user, ip";
$dbsthandle = $dbhandle->prepare($statement)
    or die "Error:  can't prepare statement \"$statement\" ($dbhandle->errstr).\n";
$dbrowvalues = $dbsthandle->execute
    or die "Error:  can't execute the query ($dbsthandle->errstr).\n";

#print "Importing Middleware records into usage database...\n";
@dbrow = $dbsthandle->fetchrow_array;
while(@mw_row = $mw_sthandle->fetchrow_array) {
    $dt = $mw_row[0];
    $user = $mw_row[1];
    $ip = $mw_row[2];
    $tool = $mw_row[3];
    $execunit = $mw_row[4];
    $walltime = $mw_row[5];

    #  If no ending datetime, mark walltime as unknown...
    #-----------------------------------------------------
    if($walltime && $walltime < 0) {
        $walltime = -1;
    }
    else {
        $walltime = int($walltime + .5);
    }

    #  Advance toolstart pointer up to Middleware ponter...
    #----------------------------------------------------
    while( @dbrow
         && ($dbrow[1] lt $dt
         || ($dbrow[1] eq $dt && $dbrow[2] lt $user)
         || ($dbrow[1] eq $dt && $dbrow[2] eq $user && $dbrow[3] lt $ip)) ) {
        @dbrow = $dbsthandle->fetchrow_array;
    }

    #  Check for an existing toolstart record...
    #--------------------------------------------
    if(@dbrow && $dbrow[1] eq $dt && $dbrow[2] eq $user && $dbrow[3] eq $ip) {

        #  If toolstart record is incomplete, check Middleware record for an update...
        #---------------------------------------------------------------------------
        if($dbrow[6] < 0 && $walltime > 0) {
            $statement = "UPDATE toolstart SET walltime = " . $dbhandle->quote($walltime)
                 . " WHERE id = " . $dbhandle->quote($dbrow[0])
        }
        else {
            $statement = "";
        }
        @dbrow = $dbsthandle->fetchrow_array;
    }
    else {

        #  If no existing toolstart record, create a new one...
        #-------------------------------------------------------
        $statement = "INSERT INTO toolstart (" .
             "datetime, success, user, ip, tool, execunit, walltime" .
             ") VALUES(" .
             $dbhandle->quote($dt) . ", " .
             "'1', " .
             $dbhandle->quote($user) . ", " .
             $dbhandle->quote($ip) . ", " .
             $dbhandle->quote($tool) . ", " .
             $dbhandle->quote($execunit) . ", " .
             $dbhandle->quote($walltime) . ")";
    }
    if($statement) {
        if($debug > 1) {
            print $statement . "\n";
        }
        elsif(!$debug) {
            $dbhandle->do($statement)
                or die "Error:  can't execute statement \"$statement\" ($dbhandle->errstr).\n";
        }
    }
}


#  Close database...
#--------------------
#print "Closing databases...\n";
$dbsthandle->finish;
$mw_sthandle->finish;
$dbhandle->disconnect;
$mw_handle->disconnect;

